##########################################################################
###
### Copyright (C) 2007 Jason D. Nielsen
###
### This program is free software# you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation# either version 2 of the License, or
### (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY# without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program# if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA
###
##########################################################################

system("make")
system("make clean")
dyn.load(paste(getwd(),"/dm_NHPP.so",sep=""))

load("cb_dat.Robj") 

dmNHPP<-function(Data,X,Panel,ng)
{
#  R wrapper for Fortran code for finding spline NHPP coef
#       Data :  nxp matrix of counts (code -99 for missing counts)
#       X    :  ntrtx1 vector containing the number of individuals in each trt
#               group.  These correspond to the rows of the Data file. For the
#               cherry bark data the first 10 rows correspond to treatment 1 and
#               the next 10 correspond to treatment 2.  So in this case X = (10,10).                                      
#       Panel:  vector (px1) of Panel times
#       ng   :  number of components desired (if you select ng > G the algorithm
#               will exit with an error, in such cases simply reduce ng=ng-1 and
#               run again). 
#   
#  Returns:  list containing est. coefs, est. functions, VarCov's and other useful
#            summary information
#
  
############## Finalize solution and compute VarCov
  vind<-TRUE
  fini<-TRUE
##############  
  n<-nrow(Data)
  p<-ncol(Data)
  if (ng > n) {
    stop("dmNHPP: # of groups > # of indiv.#")
  }
  if (p > 30) {
    nknt<-30
  }
  else {
    nknt<-length(Panel)
  }

  # randomly pick a starting point for each individual and initialize the
  # search algorithm
  pwt<-runif(ng*n)
  pwt<-matrix(pwt,n,ng)
  pwt<-pwt/apply(pwt,1,sum)
  pini<-1
  if (ng==1) pini<-0
  ntr<-length(X)
  if (sum(X)!=n) {
    stop("dmNHPP: # of indiv. in trt > than # of indiv.#")
  }
  if (length(Panel)!=p) {
    stop("dmNHPP: # of panels are panel counts don't match#")
  }
  nscf<-nknt+4
  ncf<-ntr*nscf+1
  nbg<-ncf*ng+ng-1
  Dat<-t(Data)
  cfspl<-rep(0,(ntr*(nknt+5)+1)*ng)
  VCov<-rep(0,(3*nbg)*nbg)
  jcoef<-rep(0,nbg*n)
  FortRtn<-.Fortran("r_dmnhpp",as.double(Panel),as.double(Dat),as.integer(X),coef=as.double(cfspl),
                    pwt=as.double(pwt),VCov=as.double(VCov),jcoef=as.double(jcoef),llk=as.double(0),
                    edf=as.double(0),as.integer(p),as.integer(n),as.integer(nknt),as.integer(ntr),
                    as.integer(ng),as.integer(fini),as.integer(vind),as.integer(pini),err=as.integer(0))
  if (FortRtn$err != 0 ) return(0)
  coefs<-matrix(FortRtn$coef,ncf+ntr,ng)
  VrCov<-matrix(FortRtn$VCov,3*nbg,nbg)
  JkCov<-VrCov[(nbg+1):(2*nbg),]
  osCov<-VrCov[(2*nbg+1):(3*nbg),]
  VrCov<-VrCov[1:nbg,]
  jcoef<-matrix(FortRtn$jcoef,nbg,n)
  llk<-FortRtn$llk
  edf<-FortRtn$edf
  SE<-sqrt(diag(VrCov))
  jSE<-sqrt(diag(JkCov))
  oSE<-sqrt(diag(osCov))
  probwt<-matrix(FortRtn$pwt,n,ng)
  prb<-apply(probwt,2,mean)
  if (ng>1) {
    vprb<-VrCov[(ncf*ng+1):nbg,(ncf*ng+1):nbg]
    jvprb<-JkCov[(ncf*ng+1):nbg,(ncf*ng+1):nbg]
    ovprb<-osCov[(ncf*ng+1):nbg,(ncf*ng+1):nbg]
    rp<-rep(-1,ng-1)
    vprb<-sum(rp*(vprb%*%rp))
    jvprb<-sum(rp*(jvprb%*%rp))
    ovprb<-sum(rp*(ovprb%*%rp))
    pSE<-SE[(ncf*ng+1):nbg]
    jpSE<-jSE[(ncf*ng+1):nbg]
    opSE<-oSE[(ncf*ng+1):nbg]
    prb<-rbind(prb,c(pSE,sqrt(vprb)),c(jpSE,sqrt(jvprb)),c(opSE,sqrt(ovprb)))
  }
  ntm<-200
  tm<-1:ntm/ntm
  B<-bbase(tm,0,1,nknt+1,3)
  BB<-matrix(0,ntr*ntm,ntr*nscf)
  for (i in 1:ntr) {
    BB[((i-1)*ntm+1):(i*ntm),((i-1)*nscf+1):(i*nscf)]<-B
  }
  tBB<-t(BB)
  gammacoef<-NULL
  for (i in 1:ng) {
    vV<-VrCov[((i-1)*ncf+1):(i*ncf-1),((i-1)*ncf+1):(i*ncf-1)]
    oV<-osCov[((i-1)*ncf+1):(i*ncf-1),((i-1)*ncf+1):(i*ncf-1)]
    jV<-JkCov[((i-1)*ncf+1):(i*ncf-1),((i-1)*ncf+1):(i*ncf-1)]
    gg<-cbind(BB%*%coefs[1:(ncf-1),i],sqrt(diag(BB%*%vV%*%tBB)),sqrt(diag(BB%*%jV%*%tBB)),
              sqrt(diag(BB%*%oV%*%tBB)))
    gammacoef<-cbind(gammacoef,gg)
  }   
  cSE<-matrix(SE[1:(ncf*ng)],ncf,ng)
  jcSE<-matrix(jSE[1:(ncf*ng)],ncf,ng)
  ocSE<-matrix(oSE[1:(ncf*ng)],ncf,ng)
  nms<-paste("Psi",1:ntr,sep="")
  rwnames<-paste(nms[1],1:(nknt+4),sep="")
  if (ntr > 1) {
    for (i in 2:ntr) {
      rwnames<-c(rwnames,paste(nms[i],1:(nknt+4),sep=""))
    }
  }
  rwnames<-c(rwnames,"Tau",paste("Spar",1:ntr,sep=""))
  clnames<-paste("Group",1:ng,sep="")
  colnames(coefs)<-clnames
  colnames(cSE)<-paste("SE(Gr",1:ng,")",sep="")
  colnames(jcSE)<-paste("jSE(Gr",1:ng,")",sep="")
  colnames(ocSE)<-paste("oSE(Gr",1:ng,")",sep="")
  rwnames2<-paste("Indiv",1:n,sep="")
  rownames(probwt)<-rwnames2
  colnames(probwt)<-clnames
  grnames<-paste("gam",sort(rep(1:ntr,ntm)),sep="")  
  grnames<-paste(grnames,1:ntm,sep="")
  coefs<-cbind(coefs,rbind(cSE,matrix(NA,ntr,ng)),rbind(jcSE,matrix(NA,ntr,ng)),
               rbind(ocSE,matrix(NA,ntr,ng)))
  rp<-c(rbind(1:ng,(ng+1):(2*ng),(2*ng+1):(3*ng),(3*ng+1):(4*ng)))
  out<-NULL
  if (ng >1) {
    out$coef<-coefs[,rp]
    rwnames<-c("Prob",rwnames)
    out$coef<-rbind(c(prb),out$coef)
    rownames(out$coef)<-rwnames
    rownames(gammacoef)<-grnames
    colnames(gammacoef)<-colnames(out$coef)
    out$coef<-round(out$coef,3)
    out$gamma<-round(gammacoef,3)
  }
  else {
    rownames(coefs)<-rwnames
    rownames(gammacoef)<-grnames
    out$coef<-round(coefs[,rp],3)
    colnames(gammacoef)<-colnames(out$coef)
    out$gamma<-round(gammacoef,3)    
  }
  out$probwt<-round(probwt,3)
  out$VrCov<-VrCov
  out$JkCov<-JkCov
  out$osCov<-osCov
  out$jcoef<-jcoef
  out$llk<-llk
  out$edf<-edf
  out$n<-n
  out$p<-p
  out$ntm<-ntm
  out$ng<-ng
  out$ntr<-ntr
  out$nknt<-nknt
  out$ncf<-ncf
  out$nbg<-nbg
  out$prb<-prb
  out$pcoefs<-coefs
  out
}

plot.dmNHPP<-function(out) {
#                               
#      out:  R object returned by the fitting routine dmNHPP
#   
#  Returns:  Plots estimates of overall and sub-group intensities  
#
#  
  coef<-out$coef
  gamma<-out$gamma
  probwt<-out$probwt
  VrCov<-out$VrCov
  JkCov<-out$JkCov
  osCov<-out$osCov
  prb<-out$prb
  pcoefs<-out$pcoefs
  n<-out$n
  p<-out$p
  ntm<-out$ntm
  ng<-out$ng
  ntr<-out$ntr
  nknt<-out$nknt
  ncf<-out$ncf
  nbg<-out$nbg
  tm<-1:ntm/ntm
  B<-bbase(tm,0,1,nknt+1,3)
  tm<-tm*Panel[p]  
  del<-tm[2]-tm[1]
  Lamg<-NULL
  lamg<-NULL
  for (i in 1:ntr) {
    Lamg[[i]]<-matrix(0,ntm,ng)
    lamg[[i]]<-matrix(0,ntm,ng)
  }
  for(i in 1:ng) {
    Bcoef<-matrix(pcoefs[1:(ntr*(nknt+4)),i],nknt+4,ntr)
    Bcoef<-exp(B%*%Bcoef)
    if (ntr > 1 ) {
      for (j in 2:ntr) {
        Bcoef[,j]<-Bcoef[,j]*Bcoef[,1]
      }
    }
    for (j in 1:ntr) {
      lamg[[j]][,i]<-c(Bcoef[,j])
      Lamg[[j]][,i]<-cumsum((c(0,lamg[[j]][1:(ntm-1),i])+lamg[[j]][,i])/2*del)
    }
  }
  out$lamg<-lamg
  out$Lamg<-Lamg
  if (ng > 1) {
    lamf<-NULL
    Lamf<-NULL
    for (i in 1:ntr) {
      lamf[[i]]=c(lamg[[i]]%*%matrix(prb[1,]))
      Lamf[[i]]<-cumsum((c(0,lamf[[i]][1:(ntm-1)])+lamf[[i]])/2*del)
    }
    out$lamf<-lamf
    out$Lamf<-Lamf
  }
  labG<-paste("Treatment ",1:ntr," Intensity - Group Level",sep="")
  labO<-paste("Treatment ",1:ntr," Intensity - Overall",sep="")
  labG<-c(labG[ntr],labG)
  labG<-labG[1:ntr]
  labO<-c(labO[ntr],labO)
  labO<-labO[1:ntr]
  labG[1]<-paste("Baseline Intensity - Group Level")
  labO[1]<-paste("Baseline Intensity - Overall")
  if (ng >1 ) {
    if (ntr > 1) {
      if (length(dev.list())!= (2*ntr+(ntr-1))) {
        graphics.off()
        for (i in 1:(2*ntr+(ntr-1))) dev.new()
      }
    }
    else {
      if (length(dev.list())!= (2*ntr)) {
        graphics.off()
        for (i in 1:(2*ntr)) dev.new()
      }
    }
    tkdv<-matrix(1:(2*ntr),2,ntr)+1
    ovind<-1:(nknt+4)
    vind<-ovind
    for (i in 2:ng) vind<-c(vind,(ovind+((i-1)*ncf)))
    bB<-matrix(0,ntm,(ng*(nknt+4)+ng-1))                            
    for (i in 1:ntr) {
      dev.set(tkdv[1,i])
      par(mfrow=c(2,1))
      par(bg="white")
      matplot(tm,lamg[[i]],type="l",ylab=expression(paste(lambda,"(t)")),xlab="t",main=labG[i])
      matplot(tm,Lamg[[i]],type="l",ylab=expression(paste(Lambda,"(t)")),xlab="t")
      dev.set(tkdv[2,i])
      par(mfrow=c(2,1))
      par(bg="white")
      ovind<-c(vind+((i-1)*(nknt+4)),(ncf*ng+1):nbg)
      Vrpsi<-VrCov[ovind,ovind]
      for (j in 1:ng) {
        bB[,((j-1)*(nknt+4)+1):(j*(nknt+4))]=B*c(lamg[[i]][,j])*prb[1,j]/lamf[[i]]
      }
      bB[,(ng*(nknt+4)+1):(ng*(nknt+4)+ng-1)]=(lamg[[i]][,1:(ng-1)]-c(lamg[[i]][,ng]))/lamf[[i]]
      lmvr<-sqrt(diag(bB%*%Vrpsi%*%t(bB)))
      lwlm<-lamf[[i]]*exp(-1.9599*lmvr)
      uplm<-lamf[[i]]*exp(1.9599*lmvr)
      lwLm<-cumsum((c(0,lwlm[1:(ntm-1)])+lwlm)/2*del)
      upLm<-cumsum((c(0,uplm[1:(ntm-1)])+uplm)/2*del)
      lamf[[i]]<-cbind(lamf[[i]],lwlm,uplm)
      Lamf[[i]]<-cbind(Lamf[[i]],lwLm,upLm)
      matplot(tm,lamf[[i]],type="l",ylab=expression(paste(lambda,"(t)")),xlab="t",main=labO[i],lty=c(1,3,3),col=c(1,1,1))
      matplot(tm,Lamf[[i]],type="l",ylab=expression(paste(Lambda,"(t)")),xlab="t",lty=c(1,3,3),col=c(1,1,1))
    }
    out$lamf<-lamf
    out$Lamf<-Lamf
    if (ntr > 1) {
      labT<-paste("Treatment ",1:(ntr-1)," Effects",sep="")
      for (i in 1:(ntr-1)) {
        Bcoef<-pcoefs[((nknt+4)*i+1):((i+1)*(nknt+4)),1:ng]
        Bcoef<-B%*%Bcoef
        dev.set((2*ntr)+i+1)
        matplot(tm,Bcoef,type="l",ylab=expression(paste(gamma[i],"(t)")),xlab="t",main=labT[i],ylim=c(min(Bcoef)-1,max(Bcoef)+1))
      }
    }
  }
  else {
    if (ntr > 1) {
      if (length(dev.list())!= 2*ntr-1) {
        graphics.off()
        for (i in 1:(2*ntr-1)) dev.new()
      }
    }
    else {
      if (length(dev.list())!= ntr) {
        graphics.off()
        for (i in 1:ntr) dev.new()
      }  
    }
    for (i in 1:ntr) {
      dev.set(i+1)
      par(mfrow=c(2,1))
      par(bg="white")
      Vrpsi<-VrCov[((i-1)*(nknt+4)+1):(i*(nknt+4)),((i-1)*(nknt+4)+1):(i*(nknt+4))]
      lmvr<-sqrt(diag(B%*%Vrpsi%*%t(B)))
      lwlm<-lamg[[i]]*exp(-1.9599*lmvr)
      uplm<-lamg[[i]]*exp(1.9599*lmvr)
      lwLm<-cumsum((c(0,lwlm[1:(ntm-1)])+lwlm)/2*del)
      upLm<-cumsum((c(0,uplm[1:(ntm-1)])+uplm)/2*del)
      lamg[[i]]<-cbind(lamg[[i]],lwlm,uplm)
      Lamg[[i]]<-cbind(Lamg[[i]],lwLm,upLm)
      matplot(tm,lamg[[i]],type="l",ylab=expression(paste(lambda,"(t)")),xlab="t",main=labG[i],lty=c(1,3,3),col=c(1,1,1))
      matplot(tm,Lamg[[i]],type="l",ylab=expression(paste(Lambda,"(t)")),xlab="t",lty=c(1,3,3),col=c(1,1,1))
    }
    out$lamg<-lamg
    out$Lamg<-Lamg
    if (ntr > 1) {
      labT<-paste("Treatment ",1:(ntr-1)," Effects",sep="")
      for (i in 1:(ntr-1)) {
        Bcoef<-pcoefs[((nknt+4)*i+1):((i+1)*(nknt+4)),1:ng]
        Bcoef<-B%*%Bcoef
        dev.set(ntr+1+i)
        matplot(tm,Bcoef,type="l",ylab=expression(paste(gamma[i],"(t)")),xlab="t",main=labT[i],ylim=c(min(Bcoef)-1,max(Bcoef)+1))
      }
    }
  }
}

tpower<-function(x, t, p){
  # Truncated p-th power function
  (x-t)^p*(x>t)
}

bbase <- function(x, xl, xr, ndx, deg){
# Construct a B-spline basis of degree 
  dx <- (xr - xl) / ndx
  knots <- seq(xl - deg * dx, xr + deg * dx, by = dx)
  P <- outer(x, knots, tpower, deg)
  n <- dim(P)[2]
  D <- diff(diag(n), diff = deg + 1) / (gamma(deg + 1) * dx ^ deg)
  B <- (-1) ^ (deg + 1) * P %*% t(D)
  B
}

